Unit RetainableUnit;

Interface

Uses
  SysUtils,
  Classes;

Type
  ERetainableException = Class(Exception);

  TNullable<T> = Class(TObject)
  public
    Value: T;

    constructor Create(AValue:T);
  end;

  { TNullableInt64 }

  TNullableInt64 = class(TNullable<Int64>)
  private
    function GetVal: Int64;
  published
    property Val: Int64 read GetVal;
  end;

  { TNullableAnsiString }

  TNullableAnsiString = class(TNullable<AnsiString>)
  private
    function GetVal: ansistring;
  published
    property Val: ansistring read GetVal;
  end;

  TNullableTDate = class(TNullable<TDate>);
  TNullableTDateTime = class(TNullable<TDateTime>);

  TNullableInteger = class(TNullable<Integer>);
  TNullableBoolean = class(TNullable<Boolean>);
  TNullableDouble = class(TNullable<Double>);

  TClassNullable<T{$IFDEF MSWINDOWS}: Class{$ENDIF}> = Class(TObject)
  public
    Value: T;

    constructor Create(AValue:T);
    destructor Destroy; override;
  end;

  TFloatingDateTime = TDateTime;
  TNullableTFloatingDateTime = class(TNullable<TFloatingDateTime>);

  TSimpleEnum = (seItem1,seItem2); // Used only in testing tables

  TInt64Array = Array Of Int64;

  TWeakRef = Class;
  TRetainable = Class;

  TNotRetainableException = Class(Exception);

  IRetainable = Interface(IInterface)
    ['{2E584679-9097-4ABF-B8FC-7607017C6DC0}']
    Function GetRefCount: Longint;
    Function GetWeakCount: Longint;

    Function Retain: Longint;
    Function CreateWeakRef: TWeakRef;

    Property RefCount: Longint Read GetRefCount;
    Property WeakCount: Longint Read GetWeakCount;
  End;

  TRefCounter = Class(TObject)
  Strict Private
    FRefCount: Longint;
    FWeakCount: Longint;
    FObj: TRetainable;

  Public
    Constructor Create(AObj: TRetainable);
    Function Retain: Longint;
    Function WeakRetain: Longint;

    Class Function Release(ACounter: TRefCounter): Longint;
    Class Function WeakRelease(ACounter: TRefCounter): Longint;

    Function GetRetainedObj: TRetainable;
    Function GetObj: TRetainable;

    Property RefCount: Longint Read FRefCount;
    Property WeakCount: Longint Read FWeakCount;
  End;

  { TRetainable }

  TRetainable = Class(TObject, IRetainable, IInterface)
    Strict Private
      FRefCounter: TRefCounter;
      FInitRefCounter: TRefCounter;

      Function GetRefCount: Longint;
      Function GetWeakCount: Longint;

    Strict Protected
      function VarOffset(AAddr: Pointer): NativeUInt;

    Public
      {$IFDEF RETAINABLEDEBUG}
      DebugRefCount: Boolean;
      {$ENDIF}

      {$IFNDEF PASCAL_SCRIPT}
      Class Procedure ReleaseAndNil(Var RObject);

      // Pascal reference counted interface implementation

      function QueryInterface({$IFDEF FPC_RTL}constref{$ELSE}const{$ENDIF} IID: TGUID; out Obj): HResult; {$IFDEF MSWINDOWS}stdcall;{$ELSE}cdecl;{$ENDIF}
      function _AddRef: Integer; {$IFDEF MSWINDOWS}stdcall;{$ELSE}cdecl;{$ENDIF}
      function _Release: Integer; {$IFDEF MSWINDOWS}stdcall;{$ELSE}cdecl;{$ENDIF}

      {$ENDIF}

      Constructor Create;
      Destructor Destroy; Override;
      Procedure AfterConstruction; Override;

      Function Retain: Longint; Virtual;
      Class Procedure Release(RObject: TRetainable);

      Function CreateWeakRef: TWeakRef;

      Property RefCount: Longint Read GetRefCount;
      Property WeakCount: Longint Read GetWeakCount;

    Published
  End;

  { TWeakRef }

  TWeakRef = Class(TObject)
  Strict Private
    FRefCounter: TRefCounter;

  Public
    Constructor Create(ARefCounter: TRefCounter);
    Destructor Destroy; Override;
    Function GetRetainedObj: TRetainable;   // thread safe
    Function GetIntf: IRetainable;          // thread safe
    Function GetObj: TRetainable;           // NOT thread safe
  End;

  // This list doesn't maintain refcounts, but destroy releases all objects
  TRetainableList = Class(TList)
  public
    destructor Destroy; override;
  End;

  {$IFNDEF PASCAL_SCRIPT}
  Procedure FreeNil(Var RObject);
  Procedure AllocateRecord(Out RecPtr: Pointer; Size: NativeInt);
  Function AsInterface(AObject: TRetainable): IRetainable;
  Function AsObject(AInterface: IRetainable): TRetainable; Overload;
  Function AsObject(AObject: TRetainable): TRetainable; Overload;
  Function NativeIntToPtr(ANum: NativeInt): Pointer;
  Function NativeUIntToPtr(ANum: Nativeuint): Pointer;
  Function PtrToNativeInt(APtr: Pointer): NativeInt;
  Function PtrToNativeUInt(APtr: Pointer): NativeUInt;
  Procedure ClearRecord(Out Rec; ASize: NativeInt);
  Procedure ClearArray(Out Arr; ASize: NativeInt);
  {$ENDIF}

Type
// nxsys types
{$IFNDEF CPUX64}
  size_t = LongWord;
  ssize_t = LongInt;
  time_t = LongInt;
  ptrdiff_t = LongInt;
{$ELSE}
  size_t = UInt64;
  ssize_t = Int64;
  time_t = Int64;
  ptrdiff_t = Int64;
{$ENDIF}
  Psize_t = ^size_t;
  Ptime_t = ^time_t;

Implementation

Procedure DbgOut(AStr: AnsiString);
Begin
  {$IFDEF DEBUG_STDOUT}
    WriteLn(AStr);
  {$ELSE}
  {$ENDIF}
End;

{$REGION 'TNullable<T>'}

constructor TNullable<T>.Create(AValue: T);
begin
  inherited Create;
  Value:= AValue;
end;

{$ENDREGION 'TNullable<T>'}

{$REGION 'TClassNullable<T>'}

constructor TClassNullable<T>.Create(AValue: T);
begin
  inherited Create;
  Value:= AValue;
end;

destructor TClassNullable<T>.Destroy;
begin
  FreeNil(Value);

  inherited;
end;

{ TNullableAnsiString }

function TNullableAnsiString.GetVal: ansistring;
begin
  Result := Value;
end;

{ TNullableInt64 }

function TNullableInt64.GetVal: Int64;
begin
  Result := Value;
end;

{$ENDREGION 'TClassNullable<T>'}

{$REGION 'TRefCounter'}

Constructor TRefCounter.Create(AObj: TRetainable);
Begin
  Inherited Create;

  FObj := AObj;
  FRefCount := 1;
  FWeakCount := 1;
End;

Function TRefCounter.Retain: Longint;
Begin
  Result := InterlockedIncrement(FRefCount);
End;

Function TRefCounter.WeakRetain: Longint;
Begin
  Result := InterlockedIncrement(FWeakCount);
End;

Class Function TRefCounter.Release(ACounter: TRefCounter): Longint;
Begin
  Result := InterlockedDecrement(ACounter.FRefCount);

  If Result = 0 Then
    WeakRelease(ACounter);
End;

Class Function TRefCounter.WeakRelease(ACounter: TRefCounter): Longint;
Begin
  Result := InterlockedDecrement(ACounter.FWeakCount);

  If Result = 0 Then
    ACounter.Free;
End;

Function TRefCounter.GetRetainedObj: TRetainable;
Var
  CurrentRefCount, PrevRefCount: Longint;
  {$IFDEF WEAKDEBUG}
    Iter: Longint;
  {$ENDIF}
Begin
  {$IFDEF WEAKDEBUG}
    Iter := 0;
  {$ENDIF}
  CurrentRefCount := InterlockedExchangeAdd(FRefCount, 0);
  Repeat
    If CurrentRefCount = 0 Then
    Begin
      {$IFDEF WEAKDEBUG}
        DbgOut('Weak: Trying safe get of [' + DecToHex(PtrToNativeInt(Self)) + ';' + DecToHex(PtrToNativeInt(FObj)) + '], but nil');
      {$ENDIF}
      Exit(Nil);
    End;

    PrevRefCount := CurrentRefCount;
    CurrentRefCount := InterlockedCompareExchange(FRefCount, PrevRefCount + 1, PrevRefCount);
    {$IFDEF WEAKDEBUG}
      Inc(Iter);
    {$ENDIF}
  Until
    CurrentRefCount = PrevRefCount;

  {$IFDEF WEAKDEBUG}
    DbgOut('Weak: Safe get of [' + DecToHex(PtrToNativeInt(Self)) + ';' + DecToHex(PtrToNativeInt(FObj)) + '] after ' + IntToStrA(Iter) + ' iterations, refcount = ' + IntToStrA(CurrentRefCount));
  {$ENDIF}
  Result := FObj;
End;

Function TRefCounter.GetObj: TRetainable;
Var
  CurrentRefCount: Longint;
Begin
  CurrentRefCount := InterlockedExchangeAdd(FRefCount, 0);
  If CurrentRefCount > 0 Then
  Begin
    {$IFDEF WEAKDEBUG}
      WriteLn('Weak: Unsafe get of [' + DecToHex(PtrToNativeInt(Self)) + ';' + DecToHex(PtrToNativeInt(FObj)) + '] refcount = ' + IntToStrA(CurrentRefCount));
    {$ENDIF}
    Result := FObj
  End
  Else
  Begin
    {$IFDEF WEAKDEBUG}
      WriteLn('Weak: Trying unsafe get of [' + DecToHex(PtrToNativeInt(Self)) + ';' + DecToHex(PtrToNativeInt(FObj)) + '], but Nil');
    {$ENDIF}
    Result := Nil;
  End;
End;

{$ENDREGION 'TRefCounter'}

{$REGION 'TRetainable'}

constructor TRetainable.Create;
{$IFDEF FULLDEBUG}
{$IFDEF DCC}
var
  StackList : TJclStackInfoList;
  SL: TStringList;
{$ENDIF}
{$ENDIF}
begin
  Inherited Create;
  FRefCounter := TRefCounter.Create(Self);
  FInitRefCounter := FRefCounter;

  {$IFDEF RETAINABLEDEBUG}
      If (LOG_ONLY_CLASS = '') Or (LOG_ONLY_CLASS = ClassName) Then
      Begin
        DbgOut('Create ' + AnsiString(Self.ClassName) + ' @' + DecToHex(PtrToNativeUInt(Self)));
        {$IFDEF FULLDEBUG}
        {$IFDEF DCC}
          StackList := JclCreateStackList(false,0,Caller(0,false));
          SL := TStringList.Create;
          StackList.AddToStrings(SL,true,true,true,true);
          Debug(AnsiString(SL.Text));
          SL.Free;
        {$ENDIF}
          DbgOut(AnsiString(GetCurrentCallstack(0)));
        {$ENDIF}
      End;
  {$ENDIF}
end;

Destructor TRetainable.Destroy;
{$IFDEF FULLDEBUG}
{$IFDEF DCC}
var
  StackList : TJclStackInfoList;
  SL: TStringList;
{$ENDIF}
{$ENDIF}
Begin
  {$IFDEF RETAINABLEDEBUG}
    If (LOG_ONLY_CLASS = '') Or (LOG_ONLY_CLASS = ClassName) Or DebugRefCount Then
    Begin
      DbgOut('Destroy ' + AnsiString(Self.ClassName) + ' @' + DecToHex(PtrToNativeUInt(Self)));
      {$IFDEF FULLDEBUG}
      {$IFDEF DCC}
        StackList := JclCreateStackList(false,0,Caller(0,false));
        SL := TStringList.Create;
        StackList.AddToStrings(SL,true,true,true,true);
        DbgOut(AnsiString(SL.Text));
        SL.Free;
      {$ENDIF}
        DbgOut(AnsiString(GetCurrentCallstack(0)));
      {$ENDIF}
    End;
  {$ENDIF}

  FreeNil(FInitRefCounter);

  Inherited Destroy;
End;

class procedure TRetainable.Release(RObject: TRetainable);
var
  {$IFDEF FULLDEBUG}
  {$IFDEF DCC}
  StackList : TJclStackInfoList;
  SL: TStringList;
  {$ENDIF}
  {$ENDIF}
  RCount : NativeInt;
begin

  If RObject=Nil Then
  Begin
    {$IFDEF RETAINABLEDEBUG}
      DbgOut('Trying to release NIL object');
      {$IFDEF FULLDEBUG}
      {$IFDEF DCC}
        StackList := JclCreateStackList(false,0,Caller(0,false));
        SL := TStringList.Create;
        StackList.AddToStrings(SL,true,true,true,true);
        DbgOut(AnsiString(SL.Text));
        SL.Free;
      {$ENDIF}
        DbgOut(AnsiString(GetCurrentCallstack(0)));
      {$ENDIF}
    {$ENDIF}
    Exit;
  End;

  RCount:= TRefCounter.Release(RObject.FRefCounter);

  {$IFDEF RETAINABLEDEBUG}
    If (LOG_ONLY_CLASS = '') Or (LOG_ONLY_CLASS = RObject.ClassName) Or RObject.DebugRefCount Then
    Begin
      DbgOut('Release ' +  AnsiString(RObject.ClassName) + '(' + IntToStrA(RCount) + ')' + ' @' + DecToHex(PtrToNativeUInt(RObject)));
      {$IFDEF FULLDEBUG}
      {$IFDEF DCC}
        StackList := JclCreateStackList(false,0,Caller(0,false));
        SL := TStringList.Create;
        StackList.AddToStrings(SL,true,true,true,true);
        DbgOut(AnsiString(SL.Text));
        SL.Free;
      {$ENDIF}
        DbgOut(AnsiString(GetCurrentCallstack(0)));
      {$ENDIF}
    End;
  {$ENDIF}

  If RCount = 0 Then
    RObject.Free
  Else If RCount < 0 Then
    Raise ERetainableException.Create('Calling release on already freed object ' + ClassName);
end;

Class Procedure TRetainable.ReleaseAndNil(Var RObject);
Var
  ObjTmp: TObject;
  RetTmp: TRetainable;
Begin
  ObjTmp := TObject(RObject);
  If ObjTmp = Nil Then
    Exit;

  If ObjTmp Is TRetainable Then
  Begin
    RetTmp := TRetainable(RObject);
    Pointer(RObject) := Nil;
    Release(retTmp);
  End
  Else
    Raise TNotRetainableException.Create('Non retainable object passed to ReleaseAndNil');
End;

Function TRetainable.Retain: Longint;
{$IFDEF FULLDEBUG}
{$IFDEF DCC}
var
  StackList : TJclStackInfoList;
  SL: TStringList;
{$ENDIF}
{$ENDIF}
begin
  Result := FRefCounter.Retain;

  {$IFDEF RETAINABLEDEBUG}
    If (LOG_ONLY_CLASS = '') Or (LOG_ONLY_CLASS = ClassName) Or DebugRefCount Then
    Begin
      DbgOut('Retain ' + AnsiString(ClassName) + '(' + IntToStrA(Result) + ')' + ' @' + DecToHex(PtrToNativeUInt(Self)));
      {$IFDEF FULLDEBUG}
      {$IFDEF DCC}
        StackList := JclCreateStackList(false,0,Caller(0,false));
        SL := TStringList.Create;
        StackList.AddToStrings(SL,true,true,true,true);
        DbgOut(AnsiString(SL.Text));
        SL.Free;
      {$ENDIF}
        DbgOut(AnsiString(GetCurrentCallstack(0)));
      {$ENDIF}
    End;
  {$ENDIF}
end;

Function TRetainable.CreateWeakRef: TWeakRef;
Begin
  FRefCounter.WeakRetain;
  Result := TWeakRef.Create(FRefCounter);
End;

function TRetainable.QueryInterface({$IFDEF FPC_RTL}constref{$ELSE}const{$ENDIF} IID: TGUID; out Obj): HResult; {$IFDEF MSWINDOWS}stdcall;{$ELSE}cdecl;{$ENDIF}
begin
  if GetInterface(IID, Obj) then
    Result := S_OK
  else
    Result := E_NOINTERFACE;
end;

function TRetainable._AddRef: Integer; {$IFDEF MSWINDOWS}stdcall;{$ELSE}cdecl;{$ENDIF}
begin
  Result := Retain;
end;

function TRetainable._Release: Integer; {$IFDEF MSWINDOWS}stdcall;{$ELSE}cdecl;{$ENDIF}
{$IFDEF FULLDEBUG}
{$IFDEF DCC}
var
  StackList : TJclStackInfoList;
  SL: TStringList;
{$ENDIF}
{$ENDIF}
begin
  Result := FRefCounter.Release(FRefCounter);

  {$IFDEF RETAINABLEDEBUG}
    If (LOG_ONLY_CLASS = '') Or (LOG_ONLY_CLASS = ClassName) Or DebugRefCount Then
    Begin
      DbgOut('Release ' +  AnsiString(ClassName) + '(' + IntToStrA(Result) + ')' + ' @' + DecToHex(PtrToNativeUInt(Self)));
      {$IFDEF FULLDEBUG}
      {$IFDEF DCC}
        StackList := JclCreateStackList(false,0,Caller(0,false));
        SL := TStringList.Create;
        StackList.AddToStrings(SL,true,true,true,true);
        DbgOut(AnsiString(SL.Text));
        SL.Free;
      {$ENDIF}
        DbgOut(AnsiString(GetCurrentCallstack(0)));
      {$ENDIF}
    End;
  {$ENDIF}

  If Result = 0 Then
    Free
  Else If Result < 0 Then
    Raise ERetainableException.Create('Calling release on already freed object (via autorefcount) ' + ClassName);
end;

Function TRetainable.GetRefCount: Longint;
Begin
  Result := FRefCounter.RefCount;
end;

Function TRetainable.GetWeakCount: Longint;
Begin
  Result := FRefCounter.WeakCount;
end;

function TRetainable.VarOffset(AAddr: Pointer): NativeUInt;
Begin
  Result := PtrToNativeUInt(AAddr) - PtrToNativeUInt(Self);
End;

Procedure TRetainable.AfterConstruction;
Begin
  Inherited AfterConstruction;

  FInitRefCounter := Nil;
End;

{$ENDREGION 'TRetainable'}

{$REGION 'TWeakRef'}

Constructor TWeakRef.Create(ARefCounter: TRefCounter);
Begin
  Inherited Create;
  FRefCounter := ARefCounter;
End;

Destructor TWeakRef.Destroy;
Begin
  TRefCounter.WeakRelease(FRefCounter);
  Inherited;
End;

Function TWeakRef.GetRetainedObj: TRetainable;
{$IFDEF FULLDEBUG}
{$IFDEF DCC}
var
  StackList : TJclStackInfoList;
  SL: TStringList;
{$ENDIF}
{$ENDIF}
Begin
  Result := FRefCounter.GetRetainedObj;

  {$IFDEF RETAINABLEDEBUG}
    If Result <> Nil Then
    Begin
      If (LOG_ONLY_CLASS = '') Or (LOG_ONLY_CLASS = Result.ClassName) Or Result.DebugRefCount Then
      Begin
        DbgOut('Retain ' +  AnsiString(Result.ClassName) + '(' + IntToStrA(Result.RefCount) + ')' + ' @' + DecToHex(PtrToNativeUInt(Result)));
        {$IFDEF FULLDEBUG}
        {$IFDEF DCC}
          StackList := JclCreateStackList(false,0,Caller(0,false));
          SL := TStringList.Create;
          StackList.AddToStrings(SL,true,true,true,true);
          DbgOut(AnsiString(SL.Text));
          SL.Free;
        {$ENDIF}
          DbgOut(AnsiString(GetCurrentCallstack(0)));
        {$ENDIF}
      End;
    End;
  {$ENDIF}
End;

Function TWeakRef.GetIntf: IRetainable;
Begin
  Result := AsInterface(GetRetainedObj);
End;

Function TWeakRef.GetObj: TRetainable;
Begin
  Result := FRefCounter.GetObj;
End;

{$ENDREGION 'TWeakRef'}

{$REGION 'TRetainableList'}

destructor TRetainableList.Destroy;
var i:Longint;
begin
  for i := 0 to self.Count - 1 do
  Begin
    TRetainable.Release(self[i]);
  End;

  inherited;
end;

{$ENDREGION 'TRetainableList'}

Procedure FreeNil(Var RObject);
Var
  ObjTmp: TObject;
  RetTmp: TRetainable;
Begin
  ObjTmp := TObject(RObject);
  If ObjTmp = Nil Then
    Exit;

  If ObjTmp Is TRetainable Then
  Begin
    RetTmp := TRetainable(ObjTmp);
    TRetainable.Release(RetTmp);
  End
  Else
    ObjTmp.Free;

  Pointer(RObject) := Nil;
End;

Procedure AllocateRecord(Out RecPtr: Pointer; Size: NativeInt);
Begin
  RecPtr := Nil;
  GetMem(RecPtr, Size);
  FillChar(RecPtr^, Size, 0);
End;

Function AsInterface(AObject: TRetainable): IRetainable;
Begin
  If AObject <> Nil Then
  Begin
    Result := AObject;
    TRetainable.Release(AObject);
  End
  Else
    Result := Nil;
End;

Function AsObject(AInterface: IRetainable): TRetainable;
Begin
  If AInterface <> Nil Then
  Begin
    (AInterface As TRetainable).Retain;
    Result := AInterface As TRetainable;
  End
  Else
    Result := Nil;
End;

Function AsObject(AObject: TRetainable): TRetainable;
Begin
  If AObject = Nil Then
    Exit(Nil)
  Else
    Raise ERetainableException.Create('AsObject must be called on interface, but called on ' + AObject.ClassName);
End;

Function NativeIntToPtr(ANum: NativeInt): Pointer;
Begin
  {$IFDEF MSWINDOWS}
    Result := Pointer(ANum);
  {$ELSE}
    Result := Nil;
    Inc(Result, ANum);
  {$ENDIF}
End;

Function NativeUIntToPtr(ANum: Nativeuint): Pointer;
Begin
  {$IFDEF MSWINDOWS}
    Result := Pointer(ANum);
  {$ElSE}
    Result := Nil;
    Inc(Result, ANum);
  {$ENDIF}
End;

Function PtrToNativeInt(APtr: Pointer): NativeInt;
Begin
  {$HINTS OFF}
    Result := NativeInt(APtr);
  {$HINTS ON}
End;

Function PtrToNativeUInt(APtr: Pointer): NativeUInt;
Begin
  {$HINTS OFF}
    Result := NativeUInt(APtr);
  {$HINTS ON}
End;

Procedure ClearRecord(Out Rec; ASize: NativeInt);
Begin
  {$HINTS OFF}
    FillChar(Rec, ASize, 0);
  {$HINTS ON}
End;

Procedure ClearArray(Out Arr; ASize: NativeInt);
Begin
  ClearRecord(Arr, ASize);
end;

{$IFDEF RETAINABLEDEBUG}
Initialization
  Debug_Init(DBG_FILENAME);

{$ENDIF}

End.
